@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -192,30 +192,25 @@ public:
 	// May throw an Exception on fatal error.
 	void drainFrame();
 
-	// API for the Label execution machinery. Not intended to be called
-	// directly by the user.
-	// {
-	
-	// Extract and execute the next record from the innermost frame.
-	// Does not push a new frame, executes directly in the parent's frame
-	// May throw an Exception on fatal error.
-	void callNextForked();
-	// Execute callNextForked() the current stack frame drains.
-	// Calls the tracing notifications around it.
-	// Normally used to process the forked records after a label call returns.
-	// May throw an Exception on fatal error.
-	// @param lab - Label that has created the frame. Used for the tracing
-	//     notification and error messages.
-	// @param rop - Rowop that has created the frame. Used for the tracing
-	//     notification.
-	void drainForkedFrame(const Label *lab, Rowop *rop);
-
-	// } Label execution.
-
 	// Check whether the queue is empty.
-	// @return - if no rowops in the whole queue
+	// @return - true if no rowops in the whole queue
 	bool empty() const;
 
+	// Check whether the current frame is empty.
+	// @return - true if the current frame is empty
+	bool isFrameEmpty() const
+	{
+		return innerFrame_->empty();
+	}
+
+	// Check whether the unit is in the outer frame (i.e. not
+	// in the middle of a call).
+	// @return - true if the current frame is the outer frame
+	bool isInOuterFrame() const
+	{
+		return innerFrame_ == outerFrame_;
+	}
+
 	// Get the human-readable name
 	const string &getName() const
 	{
@@ -319,17 +314,75 @@ public:
 	class Tracer : public Mtarget
 	{
 	public:
+		// The printing of the rowop structure and of the contents of the
+		// rows is split into two separate methods, allowing to redefine them
+		// separately. One controls the formatting of the call chain,
+		// the other one knows how to print the contents of the rows.
+		// The first method is expected to call the second one appropriately.
+
+		// There are two ways to specify the row printing, that may come
+		// handy in different situations:
+		// 1. Inherit and re-define the virtual method for it.
+		// 2. Pass a simple C-like function pointer, for the very simple cases.
+		// The default implementation of the virtual method just calls
+		// the C-like function pointer (if it's not NULL). If you define
+		// your own virtual method, you probably don't care about the C-like
+		// function, don't need to call it, and probably set the pointer
+		// always to NULL.
+		//
+		// This is the type of that function pointer. It must start
+		// by appending a space (since the general formatting in execute()
+		// would not normally know if there is any printing and would not
+		// put a space in front).
+		typedef void RowPrinter(string &res, const RowType *rt, const Row *row);
+	
+		// XXX eventually provide a good default row printer
+
+		// @param rp - the row printer function
+		Tracer(RowPrinter *rp = NULL);
 		virtual ~Tracer();
 
 		// The callback on some event related to rowop execution happens
 		// May throw an Exception on fatal error.
 		//
+		// Should normally call printRow() to append the row contents to
+		// the trace data.
+		//
 		// @param unit - unit from where the tracer is called
 		// @param label - label that is being called
 		// @param fromLabel - during the chained calls, the parent label, otherwise NULL
 		// @param rop - rop operation that is executed
 		// @param when - the kind of event
 		virtual void execute(Unit *unit, const Label *label, const Label *fromLabel, Rowop *rop, TracerWhen when) = 0;
+
+		// The method that appends the contents of a row to the trace string.
+		// Gets normally called by execute().
+		// The default implementation calls rowPrinter_ if it's not NULL,
+		// so the very default effect is to just do nothing.
+		// 
+		// When appending the printout, it must start by appending a space
+		// (since the general formatting in execute() would not normally know
+		// if there is any printing and would not put a space in front).
+		//
+		// @param res - the trace string, to which the row printout should be
+		//        appended (append, not replace!)
+		// @param rt - the type of the row
+		// @param row - the row to print
+		virtual void printRow(string &res, const RowType *rt, const Row *row);
+
+		// Get back the buffer of messages(with the default implementation it
+		// can also be used to add messages to the buffer).
+		// A subclass is free to redefine it in any way or just leave default.
+		virtual Erref getBuffer();
+
+		// Replace the message buffer with a clean one.
+		// The old one gets simply dereferenced, so if you have a reference, you can keep it.
+		virtual void clearBuffer();
+
+	protected:
+		RowPrinter *rowPrinter_;
+		Erref buffer_; // buffer for collecting the trace
+			// a subclass doesn't have to use it but can if it wants to
 	};
 
 	// For convenience, a concrete tracer class that collects the trace information
@@ -339,24 +392,12 @@ public:
 	{
 	public:
 		// @param verbose - if true, record all the events, otherwise only the BEGIN records
-		StringTracer(bool verbose = false);
-
-		// Get back the buffer of messages
-		// (it can also be used to add messages to the buffer)
-		Erref getBuffer() const
-		{
-			return buffer_;
-		}
-
-		// Replace the message buffer with a clean one.
-		// The old one gets simply dereferenced, so if you have a reference, you can keep it.
-		void clearBuffer();
+		StringTracer(bool verbose = false, RowPrinter *rp = NULL);
 
 		// from Tracer
 		virtual void execute(Unit *unit, const Label *label, const Label *fromLabel, Rowop *rop, TracerWhen when);
 
 	protected:
-		Erref buffer_;
 		bool verbose_;
 	};
 
@@ -366,7 +407,7 @@ public:
 	{
 	public:
 		// @param verbose - if true, record all the events, otherwise only the BEGIN records
-		StringNameTracer(bool verbose = false);
+		StringNameTracer(bool verbose = false, RowPrinter *rp = NULL);
 
 		// from Tracer
 		virtual void execute(Unit *unit, const Label *label, const Label *fromLabel, Rowop *rop, TracerWhen when);
@@ -446,6 +487,26 @@ protected:
 	// @param rop - rowop to call. It must be held by the caller until returned.
 	void callGuts(Rowop *rop);
 
+	// API for the Label execution machinery. Not intended to be called
+	// directly by the user.
+	// {
+	
+	// Extract and execute the next record from the innermost frame.
+	// Does not push a new frame, executes directly in the parent's frame
+	// May throw an Exception on fatal error.
+	void callNextForked();
+	// Execute callNextForked() the current stack frame drains.
+	// Calls the tracing notifications around it.
+	// Normally used to process the forked records after a label call returns.
+	// May throw an Exception on fatal error.
+	// @param lab - Label that has created the frame. Used for the tracing
+	//     notification and error messages.
+	// @param rop - Rowop that has created the frame. Used for the tracing
+	//     notification.
+	void drainForkedFrame(const Label *lab, Rowop *rop);
+
+	// } Label execution.
+
 protected:
 	// the scheduling queue, trays work as stack frames on it
 	// (there might be a more efficient way to do it, but for now it's good enough)
