@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -9,6 +9,7 @@
 #include <typeinfo>
 #include <type/AggregatorType.h>
 #include <type/TableType.h>
+#include <type/HoldRowTypes.h>
 #include <sched/AggregatorGadget.h>
 #include <common/Exception.h>
 
@@ -30,6 +31,14 @@ AggregatorType::AggregatorType(const AggregatorType &agg) :
 	initialized_(false)
 { }
 
+AggregatorType::AggregatorType(const AggregatorType &agg, HoldRowTypes *holder) :
+	Type(false, TT_AGGREGATOR),
+	rowType_(holder->copy(agg.rowType_)),
+	name_(agg.name_),
+	pos_(-1),
+	initialized_(false)
+{ }
+
 AggregatorType::~AggregatorType()
 { }
 
