@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -10,6 +10,7 @@
 #include <type/TableType.h>
 #include <type/GroupHandleType.h>
 #include <type/AggregatorType.h>
+#include <type/HoldRowTypes.h>
 #include <table/Index.h>
 #include <table/Table.h>
 #include <table/Aggregator.h>
@@ -44,11 +45,20 @@ IndexTypeVec::IndexTypeVec(size_t size):
 	vector<IndexTypeRef>(size)
 { }
 
-IndexTypeVec::IndexTypeVec(const IndexTypeVec &orig)
+IndexTypeVec::IndexTypeVec(const IndexTypeVec &orig, bool flat)
+{
+	if (!flat) {
+		size_t n = orig.size();
+		for (size_t i = 0; i < n; i++) 
+			push_back(IndexTypeRef(orig[i].name_, orig[i].index_->copy()));
+	}
+}
+
+IndexTypeVec::IndexTypeVec(const IndexTypeVec &orig, HoldRowTypes *holder)
 {
 	size_t n = orig.size();
 	for (size_t i = 0; i < n; i++) 
-		push_back(IndexTypeRef(orig[i].name_, orig[i].index_->copy()));
+		push_back(IndexTypeRef(orig[i].name_, orig[i].index_->deepCopy(holder)));
 }
 
 void IndexTypeVec::initialize(TableType *tabtype, IndexType *parent, Erref parentErr)
@@ -61,12 +71,12 @@ void IndexTypeVec::initialize(TableType *tabtype, IndexType *parent, Erref paren
 	size_t n = size();
 	for (size_t i = 0; i < n; i++) {
 		if (at(i).name_.empty()) {
-			parentErr->appendMsg(true, strprintf("nested index %d is not allowed to have an empty name", (int)i+1));
+			parentErr.f("nested index %d is not allowed to have an empty name", (int)i+1);
 			continue;
 		}
 		IndexType *st = (*this)[i].index_;
 		if (st == NULL) {
-			parentErr->appendMsg(true, strprintf("nested index %d '%s' reference must not be NULL", (int)i+1, at(i).name_.c_str()));
+			parentErr.f("nested index %d '%s' reference must not be NULL", (int)i+1, at(i).name_.c_str());
 			continue;
 		}
 		st->setNestPos(tabtype, parent, i);
@@ -78,7 +88,7 @@ void IndexTypeVec::initialize(TableType *tabtype, IndexType *parent, Erref paren
 	for (size_t i = 0; i < n; i++) {
 		at(i).index_->initializeNested();
 		Erref se = at(i).index_->getErrors();
-		parentErr->append(strprintf("nested index %d '%s':", (int)i+1, at(i).name_.c_str()), se);
+		parentErr.fAppend(se, "nested index %d '%s':", (int)i+1, at(i).name_.c_str());
 	}
 }
 
@@ -94,7 +104,7 @@ bool IndexTypeVec::checkDups(Erref parentErr)
 	for (size_t i = 0; i < n; i++) {
 		const string &name = at(i).name_;
 		if (known.find(name) != known.end()) {
-			parentErr->appendMsg(true, strprintf("nested index %d name '%s' is used more than once", (int)i+1, name.c_str()));
+			parentErr.f("nested index %d name '%s' is used more than once", (int)i+1, name.c_str());
 			res = false;
 		}
 		known.insert(name);
@@ -156,12 +166,23 @@ IndexType::IndexType(IndexId it) :
 	initialized_(false)
 { }
 
-IndexType::IndexType(const IndexType &orig) :
+IndexType::IndexType(const IndexType &orig, bool flat) :
 	Type(false, TT_INDEX),
-	nested_(orig.nested_),
+	nested_(orig.nested_, flat),
 	tabtype_(NULL),
 	parent_(NULL),
-	agg_(orig.agg_.isNull()? NULL : orig.agg_->copy()),
+	agg_( (flat || orig.agg_.isNull())? NULL : orig.agg_->copy()),
+	indexId_(orig.indexId_),
+	initialized_(false)
+{ 
+}
+
+IndexType::IndexType(const IndexType &orig, HoldRowTypes *holder) :
+	Type(false, TT_INDEX),
+	nested_(orig.nested_, holder),
+	tabtype_(NULL),
+	parent_(NULL),
+	agg_(orig.agg_.isNull()? NULL : orig.agg_->deepCopy(holder)),
 	indexId_(orig.indexId_),
 	initialized_(false)
 { 
@@ -296,10 +317,10 @@ void IndexType::initializeNested()
 	if (!agg_.isNull()) {
 		agg_->initialize(tabtype_, this);
 		Erref se = agg_->getErrors();
-		errors_->append(strprintf("aggregator '%s':", agg_->getName().c_str()), se);
+		errors_.fAppend(se, "aggregator '%s':", agg_->getName().c_str());
 		if (agg_->getRowType() == NULL)
-			errors_->appendMsg(true, strprintf("aggregator '%s' internal error: the result row type is not initialized",
-				agg_->getName().c_str()));
+			errors_.f("aggregator '%s' internal error: the result row type is not initialized",
+				agg_->getName().c_str());
 	}
 
 	// optimize by nullifying the empty error set
