@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -78,6 +78,12 @@ public:
 	// same internal format.
 	virtual RowType *newSameFormat(const FieldVec &fields) const = 0;
 
+	// Create a copy.
+	RowType *copy() const
+	{
+		return newSameFormat(fields_);
+	}
+
 	// from Type
 	virtual Erref getErrors() const;
 	virtual bool equals(const Type *t) const;
@@ -168,6 +174,12 @@ public:
 	// @parem row2 - another row to compare
 	// @return - true if the rows contain the same data.
 	virtual bool equalRows(const Row *row1, const Row *row2) const = 0;
+
+	// Check whether the row has no payload, i.e. all the fields in it are empty.
+	// Technically, the fields don't have to be null, any 0-length fields are
+	// considered empty.
+	// @return - true if all the fields are empty
+	virtual bool isRowEmpty(const Row *row) const = 0;
 	// }
 	
 	// {
@@ -208,6 +220,7 @@ private:
 class Rowref
 {
 public:
+	// XXX add the convenience wrappers for all the RowType's methods on rows
 	typedef Row *RowPtr;
 
 	Rowref() :
