@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2011-2012 Sergey A. Babkin.
+# (C) Copyright 2011-2013 Sergey A. Babkin.
 # This file is a part of Triceps.
 # See the file COPYRIGHT for the copyright notice and license information
 #
@@ -7,7 +7,9 @@
 
 package Triceps::JoinTwo;
 
-our $VERSION = 'v1.0.91';
+sub CLONE_SKIP { 1; }
+
+our $VERSION = 'v1.0.92';
 
 use Carp;
 
@@ -25,12 +27,19 @@ use strict;
 #    right side (default: rightTable's output label), can be used to filter
 #    out some of the input. THIS IS DANGEROUS! To preserve consistency, always
 #    filter by key field(s) only, and the same condition on the left and right.
-# leftIdxPath - array reference containing the path name of index type 
-#    in the left table used for look-up,
-#    index absolutely must be a Hash (leaf or not), not of any other kind
-# rightIdxPath - array reference containing the path name of index type 
+# leftIdxPath (optional) - array reference containing the path name of index type 
 #    in the left table used for look-up,
 #    index absolutely must be a Hash (leaf or not), not of any other kind;
+#    if not present, will be found (if possible) by the list of fields from
+#    the options "by" or "byLeft", so at least one of the explicit path or the "by"
+#    varieties must be present
+# rightIdxPath (optional) - array reference containing the path name of index type 
+#    in the left table used for look-up,
+#    index absolutely must be a Hash (leaf or not), not of any other kind;
+#    if not present, will be found (if possible) by the list of fields from
+#    the options "by" or "byLeft", so at least one of the explicit path or the "by"
+#    varieties must be present;
+#    if the options "by" and "byLeft" are not present,
 #    the number and order of fields in left and right indexes must match
 #    since indexes define the fields used for the join; the types of fields
 #    have to match exactly unless allowed by option overrideKeyTypes==1
@@ -99,8 +108,8 @@ sub new # (class, optionName => optionValue ...)
 			rightTable => [ undef, sub { &Triceps::Opt::ck_mandatory(@_); &Triceps::Opt::ck_ref(@_, "Triceps::Table") } ],
 			leftFromLabel => [ undef, sub { &Triceps::Opt::ck_ref(@_, "Triceps::Label"); } ],
 			rightFromLabel => [ undef, sub { &Triceps::Opt::ck_ref(@_, "Triceps::Label"); } ],
-			leftIdxPath => [ undef, sub { &Triceps::Opt::ck_mandatory(@_); &Triceps::Opt::ck_ref(@_, "ARRAY", "") } ],
-			rightIdxPath => [ undef, sub { &Triceps::Opt::ck_mandatory(@_); &Triceps::Opt::ck_ref(@_, "ARRAY", "") } ],
+			leftIdxPath => [ undef, sub { &Triceps::Opt::ck_ref(@_, "ARRAY", "") } ],
+			rightIdxPath => [ undef, sub { &Triceps::Opt::ck_ref(@_, "ARRAY", "") } ],
 			leftFields => [ undef, sub { &Triceps::Opt::ck_ref(@_, "ARRAY") } ],
 			rightFields => [ undef, sub { &Triceps::Opt::ck_ref(@_, "ARRAY") } ],
 			fieldsLeftFirst => [ 1, undef ],
@@ -116,6 +125,13 @@ sub new # (class, optionName => optionValue ...)
 
 	&Triceps::Opt::checkMutuallyExclusive($myname, 0, "by", $self->{by}, "byLeft", $self->{byLeft});
 
+	if (!defined $self->{by} && !defined $self->{byLeft}) {
+		for my $i ('leftIdxPath', 'rightIdxPath') {
+			Carp::confess("Option '$i' must be present if both 'by' and 'byLeft' are absent") 
+				if (!defined $self->{$i});
+		}
+	}
+
 	my $selfJoin = $self->{leftTable}->same($self->{rightTable});
 	if ($selfJoin && !defined $self->{leftFromLabel}) {
 		# one side must be fed from Pre label (but still let the user override)
@@ -155,6 +171,12 @@ sub new # (class, optionName => optionValue ...)
 	my %rightmap = $self->{rightRowType}->getFieldMapping();
 	my @rightfld = $self->{rightRowType}->getFieldNames();
 
+	if (defined $self->{byLeft}) { # override the order
+		push @{$self->{byLeft}}, "!.*"; # add the implicit no-pass-through
+		my @by = &Triceps::Fields::filterToPairs("Triceps::JoinTwo::new: option 'byLeft'", \@leftfld, $self->{byLeft});
+		$self->{by} = \@by;
+	}
+
 	# find the feed labels, compare the index definitions, check that the fields match
 	for my $side ( ("left", "right") ) {
 		if (defined $self->{"${side}FromLabel"}) {
@@ -169,6 +191,23 @@ sub new # (class, optionName => optionValue ...)
 			$self->{"${side}FromLabel"} = $self->{"${side}Table"}->getOutputLabel();
 		}
 
+		if (!defined $self->{"${side}IdxPath"}) {
+			# try to find the index by keys automatically;
+			# start by extracting one side of "by"
+			my $by = $self->{by};
+			my @idxkeys;
+			for (my $i = ($side eq "left"? 0 : 1); $i <= $#$by; $i+= 2) {
+				push @idxkeys, $by->[$i];
+			}
+
+			$self->{"${side}IdxPath"} = [ $self->{"${side}Table"}->getType()->findIndexPathForKeys(@idxkeys) ];
+			Carp::confess("The ${side}Table does not have an index that matches the key set\n  ${side} key: ("
+					. join(", ", @idxkeys) . ")\n  by: ("
+					. join(", ", @{$self->{by}}) . ")\n  ${side} table type:\n    "
+					. $self->{"${side}Table"}->getType()->print("    ") . "\n ")
+				unless $#{$self->{"${side}IdxPath"}} >= 0;
+		}
+
 		my @keys;
 		($self->{"${side}IdxType"}, @keys) = $self->{"${side}Table"}->getType()->findIndexKeyPath(@{$self->{"${side}IdxPath"}});
 		# would already confess if the index is not found
@@ -203,12 +242,6 @@ sub new # (class, optionName => optionValue ...)
 			. join(", ", @leftkeys) . ")\n  right: (" . join(", ", @rightkeys) . ")\n  ")
 		unless ($#leftkeys == $#rightkeys);
 
-	if (defined $self->{byLeft}) { # override the order
-		push @{$self->{byLeft}}, "!.*"; # add the implicit no-pass-through
-		my @by = &Triceps::Fields::filterToPairs("Triceps::JoinTwo::new: option 'byLeft'", \@leftfld, $self->{byLeft});
-		$self->{by} = \@by;
-	}
-
 	if (defined $self->{by}) { # override the order
 		Carp::confess("The count of key fields in the indexes and option '" . (defined $self->{byLeft}? "byLeft" : "by")
 				. "' does not match\n  left:  (" 
