@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2011-2012 Sergey A. Babkin.
+# (C) Copyright 2011-2013 Sergey A. Babkin.
 # This file is a part of Triceps.
 # See the file COPYRIGHT for the copyright notice and license information
 #
@@ -7,7 +7,7 @@
 
 package Triceps::Label;
 
-our $VERSION = 'v1.0.91';
+our $VERSION = 'v1.0.92';
 
 use Carp;
 
@@ -41,4 +41,29 @@ sub makeRowopArray # (self, opcode, fieldValue, ...)
 	return $rop;
 }
 
+# Make a label chained from this one.
+# Automatically picks up the row type and unit from this label.
+# The arguments are the same as for makeLabel() except that the
+# row type gets skipped.
+# Confesses on any error.
+# @param name - name of the new label
+# @param clear - the clear function
+# @param exec - the label execution function
+# @param args - arguments for the clear and exec functions
+# @return - the newly created chained label
+sub makeChained # ($self, $name, &$clear, &$exec, @args)
+{
+	confess "Use: Label::makeChained(self, name, clear, exec, ...)"
+		unless ($#_ >= 3);
+	my $self = shift;
+	my $name = shift;
+	my $clear = shift;
+	my $exec = shift;
+	my $rt = $self->getRowType() or confess "$!";
+	my $unit = $self->getUnit() or confess "$!";
+	my $lb = $unit->makeLabel($rt, $name, $clear, $exec, @_) or confess "$!";
+	$self->chain($lb);
+	return $lb;
+}
+
 1;
