@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2011-2012 Sergey A. Babkin.
+# (C) Copyright 2011-2013 Sergey A. Babkin.
 # This file is a part of Triceps.
 # See the file COPYRIGHT for the copyright notice and license information
 #
@@ -7,7 +7,7 @@
 
 package Triceps::Unit;
 
-our $VERSION = 'v1.0.91';
+our $VERSION = 'v1.0.92';
 
 use Carp;
 
@@ -163,7 +163,7 @@ sub makeLoopAround # ($self, $name, $lbFirst)
 	my $lbWrapNext = $self->makeLabel($rt, $name . ".wrapnext", undef, sub {
 		$self->setMark($mark);
 	}) or confess "$!";
-	$lbWrapNext->chain($lbFirst) or confess "$!";
+	$lbWrapNext->chain($lbFirst);
 
 	return ($lbWrapNext, $mark);
 }
